/* @(#)$Id: rk.h,v 1.5 1994/09/22 01:42:01 schwartz Exp schwartz $ */

typedef struct node {		/** variable length Markov tree node **/
    char           value;	/*  ASCII symbol value (to MAX_SET-1) */
    unsigned short count;	/*  frequency count (to max_freq)     */
    struct node    *next;	/*  alternative predictions at this k */
    struct node    *up;		/*  next k level up, eg 3 points to 4 */
} tree_node_t;

enum {
    MAX_SET = 128,		/* XXX - 256 would be better */
    TOP_K = 21,
    MAX_PREDICTION_LENGTH = 255,

    /* NODE_ALLOCATION is the chunk size we use when allocating from
       system memory.  */ 
    NODE_ALLOCATION = (256*1024 / sizeof(tree_node_t))
};

/* k ptrs into the the k contexts*/
typedef tree_node_t *context_vec_t [TOP_K + 1];
typedef char context_str_t [TOP_K + 1];
typedef char pred_str_t [MAX_PREDICTION_LENGTH+1];

struct rk {
    /* the model */
    context_str_t context;	/* last chars entered */
    context_str_t old_context;	/* ... initially newline */
    context_vec_t Buf;		/* ptrs into the model k levels  */
    context_vec_t CBuf;		/* ptrs into the model k levels  */
    char first[MAX_SET];	/* the first letter of each pred */
    char pred_set[MAX_SET];	/* flags chars already in first[]*/
    char zero_freq[MAX_SET];
    tree_node_t *free_nodes;	/* now use malloc to get at init */
    tree_node_t *root;		/* the root of the k model tree  */

    /* memory and statistics */
    int next_free;		/* index of next free node avail */
    long AllocationCount;	/* for statistics */
    long CharsInModel;		/* also for statistics */
    long FrequencySplits;	/* ditto */
    int  memoryIsFull;
    long node_limit;
    int mem_limit;

    /* output */
    int pred_number;
    pred_str_t pred_buff;

    /* parameters */
    void* (*alloc)(unsigned int);
    long psize;			/* # chars to prime from prime_file*/

    int maxk;
    int wrap_mode;
    int max_len;		/* XXX */
    int max_freq;
};

int find_first (struct rk *rk);
int init_reactive (struct rk *rk, const char* zero_vec, const char* prime_vec);
int make_a_prediction (struct rk *rk, int num, char* partial_input);
int prime_from_file (struct rk *rk, char* filename);
int show_used_nodes(struct rk *rk);
int update_the_model(struct rk *rk, char *s);
struct rk *rk_alloc (void * (*alloc)());
